<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="KatePartView" version="111" translationDomain="ktexteditor6">
<MenuBar>
  <Menu name="file" noMerge="1"><text>&amp;File</text>
    <Action name="file_save" group="save_merge" />
    <Action name="file_save_as" group="save_merge" />
    <Action name="file_save_as_with_encoding" group="save_merge" />
    <Action name="file_save_copy_as" group="save_merge" />
    <Action name="file_reload" group="revert_merge" />
    <Menu name="file_export" group="print_merge"><text>&amp;Print/Export</text>
      <Action name="file_print" group="print_merge" />
      <Action name="file_print_preview" group="print_merge" />
      <Action name="file_export_html" group="print_merge" />
    </Menu>
  </Menu>

  <Menu name="edit" noMerge="1"><text>&amp;Edit</text>
    <Action name="edit_undo" group="edit_undo_merge" />
    <Action name="edit_redo" group="edit_undo_merge" />
    <Separator group="edit_undo_merge" />
    <Action name="edit_cut" group="edit_paste_merge" />
    <Action name="edit_copy" group="edit_paste_merge" />
    <Action name="edit_copy_html" group="edit_paste_merge" />
    <Action name="edit_copy_file_location" group="edit_paste_merge" />
    <Action name="edit_paste" group="edit_paste_merge" />
    <Action name="edit_paste_selection" group="edit_paste_merge" />
    <Action name="edit_paste_from_file" group="edit_paste_merge" />
    <Action name="edit_swap_with_clipboard" group="edit_paste_merge" />
    <Action name="clipboard_history_paste" group="edit_paste_merge" />/>
    <Separator group="edit_select_merge" />
    <Action name="view_input_modes" group="edit_select_merge" />
    <Action name="set_insert" group="edit_select_merge" />
    <Action name="tools_toggle_write_lock" group="edit_select_merge" />
    <Separator group="edit_select_merge" />
    <Action name="edit_find" group="edit_find_merge" />
    <Menu name="edit_find_menu" group="edit_find_merge"><text>Find Variants</text>
      <Action name="edit_find_next" group="edit_find_merge" />
      <Action name="edit_find_prev" group="edit_find_merge" />
      <Action name="edit_find_selected" group="edit_find_merge" />
      <Action name="edit_find_selected_backwards" group="edit_find_merge" />
    </Menu>
    <Action name="edit_replace" group="edit_find_merge" />
    <Separator group="edit_find_merge" />
  </Menu>

  <Menu name="selection" noMerge="1"><text>Selection</text>
    <Action name="edit_select_all" group="edit_selection" />
    <Action name="edit_deselect" group="edit_selection" />
    <Action name="set_verticalSelect" group="edit_selection" />
    <Separator group="edit_selection"/>
    <Action name="tools_toggle_comment" group="edit_selection" />
    <Action name="tools_join_lines" group="edit_selection" />
    <Menu name="capitalization" group="edit_selection"><text>Capitalization</text>
      <Action name="tools_uppercase" group="edit_selection" />
      <Action name="tools_lowercase" group="edit_selection" />
      <Action name="tools_capitalize" group="edit_selection" />
    </Menu>
    <Separator group="edit_selection"/>
    <Action name="tools_cleanIndent" group="edit_selection" />
    <Action name="tools_convertTabsSpaces" group="edit_selection" />
    <Action name="tools_convertSpacesTabs" group="edit_selection" />
    <Action name="tools_formatIndent" group="edit_selection" />
    <Action name="tools_alignOn" group="edit_selection" />
    <Action name="tools_apply_wordwrap" group="edit_selection" />
    <Separator group="edit_selection"/>
    <Action name="edit_create_multi_cursor_up" group="edit_selection"/>
    <Action name="edit_create_multi_cursor_down" group="edit_selection"/>
    <Action name="edit_create_multi_cursor_from_sel" group="edit_selection"/>
    <Action name="edit_find_multicursor_next_occurrence" group="edit_selection"/>
    <Action name="edit_find_multicursor_all_occurrences" group="edit_selection"/>
  </Menu>

  <Menu name="view" noMerge="1"><text>&amp;View</text>
    <Action name="view_inc_font_sizes" group="view_operations" />
    <Action name="view_dec_font_sizes" group="view_operations" />
    <Action name="view_reset_font_sizes" group="view_operations" />
    <Separator group="view_operations" />
    <Menu name="view_menu_word_wrap" group="view_operations"><text>Word Wrap</text>
      <Action name="view_dynamic_word_wrap" group="view_operations" />
      <Action name="dynamic_word_wrap_indicators" group="view_operations" />
      <Action name="view_static_word_wrap" group="view_operations" />
      <Action name="view_word_wrap_marker" group="view_operations" />
    </Menu>
    <Menu name="view_menu_borders" group="view_operations"><text>Borders</text>
      <Action name="view_border" group="view_operations" />
      <Action name="view_line_numbers" group="view_operations" />
      <Action name="view_scrollbar_marks" group="view_operations" />
      <Action name="view_scrollbar_minimap" group="view_operations" />
      <Action name="view_scrollbar_minimap_all" group="view_operations" />
    </Menu>
    <Separator group="view_operations" />
    <Menu name="codefolding" group="view_operations"><text>&amp;Code Folding</text>
      <Action name="view_folding_markers" group="view_operations" />
      <Separator group="view_operations" />
      <Action name="folding_toggle_current" group="view_operations" />
      <Action name="folding_toggle_in_current" group="view_operations" />
      <Action name="folding_expandall" group="view_operations" />
      <Separator group="view_operations" />
      <Action name="folding_toplevel" group="view_operations" />
      <Action name="folding_expandtoplevel" group="view_operations" />
    </Menu>
    <Separator group="view_operations" />
    <Action name="view_auto_reload" group="view_operations" />
    <Action name="view_non_printable_spaces" group="view_operations" />
    <Action name="force_rtl_direction" group="edit_select_merge" />
    <Action name="view_word_count" group="view_operations" />
  </Menu>

  <Menu name="go" noMerge="1"><text>&amp;Go</text>
    <Action name="go_goto_line" group="edit_goto"/>
    <Separator group="edit_goto" />
    <Action name="Previous Editing Line" group="edit_goto2"/>
    <Action name="Next Editing Line" group="edit_goto2"/>
    <Separator group="edit_goto2" />
    <Action name="modified_line_up" group="edit_goto2"/>
    <Action name="modified_line_down" group="edit_goto2"/>
    <Separator group="edit_goto2" />
    <Action name="to_matching_bracket" group="edit_goto2" />
    <Action name="select_matching_bracket" group="edit_goto2" />
    <Separator group="edit_goto2" />
    <Action name="bookmarks" group="edit_goto2" />
  </Menu>

  <Menu name="tools" noMerge="1"><text>&amp;Tools</text>
    <Action name="tools_mode" group="tools_operations" />
    <Action name="tools_highlighting" group="tools_operations" />
    <Action name="tools_indentation" group="tools_operations" />
    <Separator group="tools_operations" />
    <Action name="set_encoding" group="tools_operations" />
    <Action name="add_bom" group="tools_operations" />
    <Action name="set_eol" group="tools_operations" />
    <Separator group="tools_operations" />
    <Action name="tools_scripts" group="tools_operations2" />
    <Separator group="tools_operations2" />
    <Action name="switch_to_cmd_line" group="tools_operations2" />
    <Separator group="tools_operations2" />
    <Action name="tools_invoke_code_completion" group="tools_operations2" />
    <Menu name="wordcompletion" group="tools_operations2"><text>Word Completion</text>
      <Action name="doccomplete_fw" />
      <Action name="doccomplete_bw" />
      <Action name="doccomplete_sh" />
    </Menu>
    <Separator group="tools_operations2" />
    <Menu name="spelling" group="tools_spelling"><text>Spelling</text>
      <Action name="tools_toggle_automatic_spell_checking" group="tools_spelling" />
      <Action name="tools_spelling" group="tools_spelling" />
      <Action name="tools_spelling_from_cursor" group="tools_spelling" />
      <Action name="tools_spelling_selection" group="tools_spelling" />
      <Action name="tools_change_dictionary" group="tools_spelling" />
      <Action name="tools_clear_dictionary_ranges" group="tools_spelling" />
    </Menu>
    <Separator group="tools_operations2" />
    <Menu name="speech" group="tools_speech"><text>Text to Speech</text>
      <Action name="tools_speech_say" group="tools_speech" />
      <Action name="tools_speech_stop" group="tools_speech" />
      <Action name="tools_speech_pause" group="tools_speech" />
      <Action name="tools_speech_resume" group="tools_speech" />
    </Menu>
  </Menu>

  <Menu name="settings" noMerge="1"><text>&amp;Settings</text>
    <Action name="view_schemas" group="color" />
    <Action name="set_confdlg" group="configure_merge" />
  </Menu>
</MenuBar>

<Menu name="ktexteditor_popup" noMerge="0">
  <Action name="spelling_suggestions" group="popup_operations" />
  <Separator group="popup_operations" />
  <Action name="edit_cut" group="popup_operations" />
  <Action name="edit_copy" group="popup_operations" />
  <Action name="edit_copy_html" group="popup_operations" />
  <Action name="edit_paste" group="popup_operations" />
  <Action name="edit_paste_selection" group="popup_operations" />
  <Action name="edit_swap_with_clipboard" group="popup_operations" />
  <Action name="text_screenshot_selection" group="popup_operations" />
  <Separator group="popup_operations" />
  <Action name="tools_scripts_Editing" group="popup_operations" />
  <Separator group="popup_operations" />
  <Action name="bookmarks" group="popup_operations" />
  <Separator group="popup_operations" />
  <Action name="tools_create_snippet" group="popup_operations" />
  <Separator group="popup_operations" />
  <Menu name="speech" group="popup_operations"><text>Text to Speech</text>
    <Action name="tools_speech_stop" group="popup_operations" />
    <Action name="tools_speech_pause" group="popup_operations" />
    <Action name="tools_speech_resume" group="popup_operations" />
  </Menu>
  <Action name="tools_speech_say" group="popup_operations" />
</Menu>

<ToolBar name="mainToolBar" noMerge="1"><text>Main Toolbar</text>
  <Action name="file_save" group="file_operations" />
  <Action name="file_save_as" group="file_operations" />
  <Action name="edit_undo" group="edit_operations" />
  <Action name="edit_redo" group="edit_operations" />
</ToolBar>

</gui>
